@@ -1,16 +1,13 @@
-{-# LANGUAGE FlexibleContexts       #-}
+{-# LANGUAGE DeriveGeneric          #-}
 {-# LANGUAGE FlexibleInstances      #-}
 {-# LANGUAGE FunctionalDependencies #-}
 {-# LANGUAGE MultiParamTypeClasses  #-}
 {-# LANGUAGE PostfixOperators       #-}
 {-# LANGUAGE Safe                   #-}
-{-# LANGUAGE TypeSynonymInstances   #-}
 
 {-# OPTIONS_GHC -fno-warn-missing-signatures #-}
 {-# OPTIONS_GHC -fno-warn-name-shadowing #-}
 
-{-# OPTIONS_GHC -fno-warn-incomplete-patterns #-} -- FIXME
-
 -- |
 -- Copyright: © Oren Ben-Kiki 2007,
 --            © Herbert Valerio Riedel 2015-2018
@@ -22,6 +19,7 @@ module Data.YAML.Token
   ( tokenize
   , Token(..)
   , Code(..)
+  , Encoding(..)
   ) where
 
 import qualified Data.ByteString.Lazy.Char8 as BLC
@@ -126,7 +124,11 @@ data Code = Bom             -- ^ BOM, contains \"@TF8@\", \"@TF16LE@\", \"@TF32B
           | Error           -- ^ Parsing error at this point.
           | Unparsed        -- ^ Unparsed due to errors (or at end of test).
           | Detected        -- ^ Detected parameter (for testing).
-  deriving (Show,Eq)
+  deriving (Show,Eq,Generic)
+
+-- | @since 0.2.0
+instance NFData Code where
+  rnf x = seq x ()
 
 {-
 -- | @show code@ converts a 'Code' to the one-character YEAST token code char.
@@ -185,8 +187,11 @@ data Token = Token {
     tLineChar   :: !Int,   -- ^ 0-based character in line.
     tCode       :: !Code,  -- ^ Specific token 'Code'.
     tText       :: !String -- ^ Contained input chars, if any.
-  } deriving Show
+  } deriving (Show,Generic)
 
+-- | @since 0.2.0
+instance NFData Token where
+  rnf Token { tText = txt } = rnf txt
 
 -- * Parsing framework
 --
@@ -208,7 +213,7 @@ data Token = Token {
 -- so we maintain a single 'State' type rather than having a generic one that
 -- contains a polymorphic \"UserState\" field etc.
 
--- | A 'Parser' is basically a function computing a 'Reply'.
+-- | A 'Data.YAML.Token.Parser' is basically a function computing a 'Reply'.
 newtype Parser result = Parser (State -> Reply result)
 
 applyParser :: Parser result -> State -> Reply result
@@ -220,14 +225,16 @@ data Result result = Failed String        -- ^ Parsing aborted with a failure.
                    | Result result        -- ^ Parsing completed with a result.
                    | More (Parser result) -- ^ Parsing is ongoing with a continuation.
 
+{-
 -- Showing a 'Result' is only used in debugging.
 instance (Show result) => Show (Result result) where
   show result = case result of
                      Failed message -> "Failed " ++ message
                      Result result  -> "Result " ++ (show result)
                      More _         -> "More"
+-}
 
--- | Each invication of a 'Parser' yields a 'Reply'. The 'Result' is only one
+-- | Each invocation of a 'Data.YAML.Token.Parser' yields a 'Reply'. The 'Result' is only one
 -- part of the 'Reply'.
 data Reply result = Reply {
     rResult :: !(Result result), -- ^ Parsing result.
@@ -236,12 +243,14 @@ data Reply result = Reply {
     rState  :: !State            -- ^ The updated parser state.
   }
 
+{-
 -- Showing a 'State' is only used in debugging.
 instance (Show result) => Show (Reply result) where
   show reply = "Result: "    ++ (show $ reply^.rResult)
             ++ ", Tokens: "  ++ (show $ D.toList $ reply^.rTokens)
             ++ ", Commit: "  ++ (show $ reply^.rCommit)
             ++ ", State: { " ++ (show $ reply^.rState) ++ "}"
+-}
 
 -- A 'Pattern' is a parser that doesn't have an (interesting) result.
 type Pattern = Parser ()
@@ -272,6 +281,7 @@ data State = State {
     sInput           :: ![(Int, Char)]    -- ^ The decoded input characters.
   }
 
+{-
 -- Showing a 'State' is only used in debugging. Note that forcing dump of
 -- @sInput@ will disable streaming it.
 instance Show State where
@@ -292,6 +302,7 @@ instance Show State where
             ++ ", Code: "            ++ (show $ state^.sCode)
             ++ ", Last: "            ++ (show $ state^.sLast)
 --          ++ ", Input: >>>"        ++ (show $ state^.sInput) ++ "<<<"
+-}
 
 -- | @initialState name input@ returns an initial 'State' for parsing the
 -- /input/ (with /name/ for error messages).
@@ -327,14 +338,17 @@ initialState input
 -- | @setLimit limit state@ sets the @sLimit@ field to /limit/.
 setLimit :: Int -> State -> State
 setLimit limit state = state { sLimit = limit }
+{-# INLINE setLimit #-}
 
 -- | @setForbidden forbidden state@ sets the @sForbidden@ field to /forbidden/.
 setForbidden :: Maybe Pattern -> State -> State
 setForbidden forbidden state = state { sForbidden = forbidden }
+{-# INLINE setForbidden #-}
 
 -- | @setCode code state@ sets the @sCode@ field to /code/.
 setCode :: Code -> State -> State
 setCode code state = state { sCode = code }
+{-# INLINE setCode #-}
 
 -- ** Implicit parsers
 --
@@ -343,11 +357,11 @@ setCode code state = state { sCode = code }
 -- 'Match' class.
 
 -- | @Match parameter result@ specifies that we can convert the /parameter/ to
--- a 'Parser' returning the /result/.
+-- a 'Data.YAML.Token.Parser' returning the /result/.
 class Match parameter result | parameter -> result where
     match :: parameter -> Parser result
 
--- | We don't need to convert a 'Parser', it already is one.
+-- | We don't need to convert a 'Data.YAML.Token.Parser', it already is one.
 instance Match (Parser result) result where
     match = id
 
@@ -484,17 +498,17 @@ infix  0 >!
 -- | @parser % n@ repeats /parser/ exactly /n/ times.
 (%) :: (Match match result) => match -> Int -> Pattern
 parser % n
-  | n <= 0 = empty
-  | n >  0 = parser' *> (parser' % n .- 1)
+  | n <= 0    = empty
+  | otherwise = parser' *> (parser' % n .- 1)
   where
     parser' = match parser
 
 -- | @parser <% n@ matches fewer than /n/ occurrences of /parser/.
 (<%) :: (Match match result) => match -> Int -> Pattern
-parser <% n
-  | n < 1 = pfail "Fewer than 0 repetitions"
-  | n == 1 = reject parser Nothing
-  | n > 1  = DeLess ^ ( ((parser ! DeLess) *> (parser <% n .- 1)) <|> empty )
+parser <% n = case n `compare` 1 of
+  LT -> pfail "Fewer than 0 repetitions"
+  EQ -> reject parser Nothing
+  GT -> DeLess ^ ( ((parser ! DeLess) *> (parser <% n .- 1)) <|> empty )
 
 data Decision = DeNone -- ""
               | DeStar -- "*"
@@ -695,7 +709,7 @@ empty = return ()
 -- | @eof@ matches the end of the input.
 eof :: Pattern
 eof = Parser $ \ state ->
-  if state^.sInput == []
+  if null (state^.sInput)
      then returnReply state ()
      else unexpectedReply state
 
@@ -741,6 +755,7 @@ with setField getField value parser = Parser $ \ state ->
              Failed _     -> reply { rState = setField parentValue $ reply^.rState }
              Result _     -> reply { rState = setField parentValue $ reply^.rState }
              More parser' -> reply { rResult = More $ withParser parentValue parser' }
+{-# INLINE with #-}
 
 -- | @parser ``forbidding`` pattern@ parses the specified /parser/ ensuring
 -- that it does not contain anything matching the /forbidden/ parser.
@@ -765,6 +780,7 @@ nextIf test = Parser $ \ state ->
                         in case reply^.rResult of
                                 Failed _ -> reply
                                 Result _ -> limitedNextIf state
+                                More _   -> error "unexpected Result More _ pattern"
   where
     limitedNextIf state =
         case state^.sLimit of
@@ -797,7 +813,7 @@ nextIf test = Parser $ \ state ->
       where
         charsOf field charsField
           | state^.sIsPeek       = -1
-          | state^.sChars == []  = state^.field
+          | null (state^.sChars) = state^.field
           | otherwise            = state^.charsField
 
 -- ** Producing tokens
@@ -910,6 +926,7 @@ data Context = BlockOut     -- ^ Outside block sequence.
              | BlockKey     -- ^ Implicit block key.
              | FlowKey      -- ^ Implicit flow key.
 
+{-
 -- | @show context@ converts a 'Context' to a 'String'.
 instance Show Context where
   show context = case context of
@@ -933,12 +950,14 @@ instance Read Context where
                         "block_key" -> BlockKey
                         "flow_key"  -> FlowKey
                         _           -> error $ "unknown context: " ++ word
+-}
 
 -- | Chomp method.
 data Chomp = Strip -- ^ Remove all trailing line breaks.
            | Clip  -- ^ Keep first trailing line break.
            | Keep  -- ^ Keep all trailing line breaks.
 
+{-
 -- | @show chomp@ converts a 'Chomp' to a 'String'.
 instance Show Chomp where
   show chomp = case chomp of
@@ -954,10 +973,11 @@ instance Read Chomp where
                         "clip"  -> Clip
                         "keep"  -> Keep
                         _       -> error $ "unknown chomp: " ++ word
+-}
 
 -- * Tokenizers
 --
--- We encapsulate the 'Parser' inside a 'Tokenizer'. This allows us to hide the
+-- We encapsulate the 'Data.YAML.Token.Parser' inside a 'Tokenizer'. This allows us to hide the
 -- implementation details from our callers.
 
 -- | 'Tokenizer' converts a input text into a list of 'Token'. Errors
@@ -1059,7 +1079,7 @@ asInteger = Parser $ \ state -> returnReply state $ ord (state^.sLast) .- 48
 
 -- | @result value@ is the same as /return value/ except that we give the
 -- Haskell type deduction the additional boost it needs to figure out this is
--- wrapped in a 'Parser'.
+-- wrapped in a 'Data.YAML.Token.Parser'.
 result :: result -> Parser result
 result = return
 
@@ -1216,6 +1236,7 @@ s_line_prefix n c {- 67 -} = case c of
                                   BlockIn  -> s_block_line_prefix n
                                   FlowOut  -> s_flow_line_prefix n
                                   FlowIn   -> s_flow_line_prefix n
+                                  _        -> error "unexpected node style pattern in s_line_prefix"
 
 s_block_line_prefix n {- 68 -} = s_indent n
 s_flow_line_prefix  n {- 69 -} = s_indent n & ( s_separate_in_line ?)
@@ -1366,6 +1387,8 @@ nb_double_text n c  {- 110 -} = case c of
                                      FlowIn   -> nb_double_multi_line n
                                      BlockKey -> nb_double_one_line
                                      FlowKey  -> nb_double_one_line
+                                     _        -> error "unexpected node style pattern in nb_double_text"
+
 nb_double_one_line  {- 111 -} = ( nb_double_char *)
 
 s_double_escaped n {- 112 -} = ( s_white *)
@@ -1395,6 +1418,8 @@ nb_single_text n c {- 121 -} = case c of
                                     FlowIn   -> nb_single_multi_line n
                                     BlockKey -> nb_single_one_line
                                     FlowKey  -> nb_single_one_line
+                                    _        -> error "unexpected node style pattern in nb_single_text"
+
 nb_single_one_line {- 122 -} = ( nb_single_char *)
 
 nb_ns_single_in_line    {- 123 -} = ( ( s_white *) & ns_single_char *)
@@ -1414,7 +1439,8 @@ ns_plain_safe c   {- 127 -} = case c of
                                    FlowIn   -> ns_plain_safe_in
                                    BlockKey -> ns_plain_safe_out
                                    FlowKey  -> ns_plain_safe_in
-                                   
+                                   _        -> error "unexpected node style pattern in ns_plain_safe"
+
 ns_plain_safe_out {- 128 -} = ns_char
 ns_plain_safe_in  {- 129 -} = ns_char - c_flow_indicator
 ns_plain_char c   {- 130 -} = ns_plain_safe c - ':' - '#'
@@ -1426,7 +1452,9 @@ ns_plain n c          {- 131 -} = wrapTokens BeginScalar EndScalar
                                              FlowOut  -> ns_plain_multi_line n c
                                              FlowIn   -> ns_plain_multi_line n c
                                              BlockKey -> ns_plain_one_line c
-                                             FlowKey  -> ns_plain_one_line c)
+                                             FlowKey  -> ns_plain_one_line c
+                                             _        -> error "unexpected node style pattern in ns_plain")
+
 nb_ns_plain_in_line c {- 132 -} = ( ( s_white *) & ns_plain_char c *)
 ns_plain_one_line c   {- 133 -} = ns_plain_first c ! DeNode & nb_ns_plain_in_line c
 
@@ -1442,6 +1470,7 @@ in_flow c {- 136 -} = case c of
                            FlowIn   -> FlowIn
                            BlockKey -> FlowKey
                            FlowKey  -> FlowKey
+                           _        -> error "unexpected node style pattern in in_flow"
 
 -- 7.4.1 Flow Sequences
 
@@ -1473,9 +1502,9 @@ ns_flow_map_explicit_entry n c {- 143 -} = ns_flow_map_implicit_entry n c
                                            & e_node )
 
 ns_flow_map_implicit_entry n c {- 144 -}    = DePair
-                                            ^ ( ns_flow_map_yaml_key_entry n c
-                                              / c_ns_flow_map_empty_key_entry n c
-                                              / c_ns_flow_map_json_key_entry n c )
+                                            ^ ( c_ns_flow_map_json_key_entry n c
+                                              / ns_flow_map_yaml_key_entry n c
+                                              / c_ns_flow_map_empty_key_entry n c )
 ns_flow_map_yaml_key_entry n c {- 145 -}    = ( DeNode ^ ns_flow_yaml_node n c ) ! DePair
                                             & ( ( ( s_separate n c ?)
                                                 & c_ns_flow_map_separate_value n c )
@@ -1669,7 +1698,7 @@ ns_l_block_map_entry n {- 188 -} = wrapTokens BeginPair EndPair
                                  / ns_l_block_map_implicit_entry n
 c_l_block_map_explicit_entry n {- 189 -} = c_l_block_map_explicit_key n
                                          & ( l_block_map_explicit_value n
-                                         / e_node )                                         
+                                         / e_node )
 c_l_block_map_explicit_key n   {- 190 -} = c_mapping_key & ( ns_char >!) ! DeNode & s_l__block_indented n BlockOut
 l_block_map_explicit_value n   {- 191 -} = s_indent n & c_mapping_value & s_l__block_indented n BlockOut
 
@@ -1713,6 +1742,7 @@ s_l__block_collection n c {- 200 -} = ( s_separate (n .+ 1) c & c_ns_properties
 seq_spaces n c            {- 201 -} = case c of
                                            BlockOut -> n .- 1
                                            BlockIn  -> n
+                                           _        -> error "unexpected node style pattern in seq_spaces"
 
 -- 9.1.1 Document Prefix
 
